/*   FILE: db-generic.c -- Build a media database
 * AUTHOR: W. Michael Petullo <mike@flyn.org>
 *   DATE: 09 July 2009 
 *
 * Copyright (c) 2009 W. Michael Petullo <new@flyn.org>
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "db.h"

static gint
add_file_to_db (const char *path, DMAPDb *db, DMAPRecordFactory *factory)
{
	gint id = 0;
	DMAPRecord *record = dmap_record_factory_create (factory, g_strdup_printf ("%s", path));
	if (! record) {
		g_warning ("Could not load %s", path);
	} else {
		id = dmap_db_add (db, record);
	}

	return id;
}

void
build_db_starting_at (const char *dir,
			   DMAPDb *db,
			   DMAPContainerDb *container_db,
			   DMAPContainerRecord *container_record,
			   DMAPRecordFactory *factory)
{
	GError *error = NULL;

	GDir *d = g_dir_open (dir, 0, &error);

	if (error != NULL) {
		g_error ("Error opening %s", dir);
	} else {
		const gchar *entry;
		
		while (entry = g_dir_read_name (d)) {
			gchar *path = g_strdup_printf ("%s/%s", dir, entry);

			if (g_file_test (path, G_FILE_TEST_IS_DIR)) {
				DMAPContainerRecord *record = DMAP_CONTAINER_RECORD (g_object_new (TYPE_DMAPD_DMAP_CONTAINER_RECORD, "name", g_strdup (entry), NULL));
				build_db_starting_at (path, db, container_db, record, factory);
				dmapd_dmap_container_db_add (container_db, record);
			} else {
				gint id;

				id = add_file_to_db (path, db, factory);
				if (id) {
					DMAPRecord *record = dmap_db_lookup_by_id (db, id);
					if (container_record) {
						dmap_container_record_add_entry (container_record, record, id);
					}
				}
			}
			g_free (path);
		}
		
		g_dir_close (d);
	}
}
